/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.algo.SilentConsoleStatusBar;
import de.timefinder.algo.io.DataPool4Track2;
import de.timefinder.algo.ncp.NoCollisionPrinciple;
import de.timefinder.algo.ncp.ConstraintChecker;
import de.timefinder.algo.ncp.NCPSettings;
import de.timefinder.algo.set.SetFactoryImpl;
import de.timefinder.data.DataPool;
import de.timefinder.data.DataPoolImpl;
import de.timefinder.data.access.EventDao;
import de.timefinder.data.access.FeatureDao;
import de.timefinder.data.access.LocationDao;
import de.timefinder.data.access.PersonDao;
import de.timefinder.data.access.impl.EventDaoListImpl;
import de.timefinder.data.access.impl.FeatureDaoListImpl;
import de.timefinder.data.access.impl.LocationDaoListImpl;
import de.timefinder.data.access.impl.PersonDaoListImpl;
import de.timefinder.data.set.SetFactory;
import de.timefinder.core.io.xml.bt.ImportBTXml;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.logging.Logger;

/**
 * Intel Core Duo P8600 @ 2.40GHz; 4 GB RAM; Ubuntu 9.04
 * (find out with "sudo lshw -html > hw.html; opera hw.html")
 *
 * 1. close all applications, the IDE inclusive and do not do anything while the run!
 * 2. set variables COUNT = 100 and NCP_COUNT = 75 iterations per run
 *    TODO more iterations than 75 introduce unpredictable results for tim 3
 * 3. set log level to WARNING for core.algo in ~/.timefinder/log.properties
 * 4. be sure that your laptop mode is *performance* (maybe force it via running one time ...)
 * 5. run via mvn exec:java -Dexec.mainClass=de.timefinder.core.algo.ncp.RunAlgo
 *
 *
 * 60 sec
 *
 * tim 09 => counter=1292 (hc==0; sc==2462)
 * tim 08 => counter=3750 (hc==0; sc== 839)
 * tim 03 => counter=3797 (hc==0; sc==1848)
 *
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class RunAlgo {

    private static Logger logger = Logger.getLogger(RunAlgo.class.getName());
    private static String TIM_FILE = "/home/peterk/Dokumente/timefinder/ITC-2007-2008/Track2/comp-2007-2-10.tim";
    private static String BT_FOLDER = "/home/peterk/Dokumente/timefinder/uni-bt";
    private static String OPTIMIZATION_FILE;
    private static boolean LOAD_BT = true;

    public static void main(String args[]) throws Exception {
        Startup.initLogging();

        if (LOAD_BT)
            OPTIMIZATION_FILE = BT_FOLDER;
        else
            OPTIMIZATION_FILE = TIM_FILE;

        new RunAlgo().start(8000);

//        for (int i = 1; i < 10; i++) {
//            OPTIMIZATION_FILE = "/home/peterk/Dokumente/timefinder/ITC-2007-2008/Track2/comp-2007-2-0" + i + ".tim";
//            new RunAlgo().start(50);
//        }
//
//        for (int i = 10; i < 17; i++) {
//            OPTIMIZATION_FILE = "/home/peterk/Dokumente/timefinder/ITC-2007-2008/Track2/comp-2007-2-" + i + ".tim";
//            new RunAlgo().start(50);
//        }

        // warm up
//        new RunAlgo().start(500);
//        new RunAlgo().start(500);
//        new RunAlgo().start(500);

//        for (int i = 1; i < 10; i++) {
//            FeatureSet.clearStaticStorage();
//            long tmpTime = new RunAlgo().start(-1, TIM_FOLDER + "comp-2007-2-0" + i + ".tim");
//            mem(0, tmpTime);
//        }
//
//        for (int i = 10; i < 17; i++) {
//            FeatureSet.clearStaticStorage();
//            long tmpTime = new RunAlgo().start(-1, TIM_FOLDER + "comp-2007-2-" + i + ".tim");
//            mem(0, tmpTime);
//        }
    }

    public void performance() throws Exception {
        int NCP_COUNTER = 100;
        int COUNT = 10;

        logger.info("Start initialization");
        long tmpTime;
        // do not count the initial warm up phase
        for (int ii = 0; ii < COUNT / 4; ii++) {
            tmpTime = start(NCP_COUNTER);
            mem(ii, tmpTime);
        }

        logger.info("Start measurement");
        List<Long> allDuration = new ArrayList<Long>();
        for (int ii = 0; ii < COUNT; ii++) {
            tmpTime = start(NCP_COUNTER);
            mem(ii, tmpTime);
            allDuration.add(tmpTime);
        }

        statistics(allDuration);
    }

    public static void mem(int i, long time) {
        System.out.println("iteration " + i + "; time was: " + time +
                ";  Free Mem " + (float) (Runtime.getRuntime().freeMemory() / 1024.0 / 1024.0) + " MB");
    }

    private static void statistics(List<Long> allDurations) {
        long timeSum = 0;
        long maxDuration = 0;
        long minDuration = Integer.MAX_VALUE;

        for (Long val : allDurations) {
            if (val > maxDuration) {
                maxDuration = val;
            }
            if (val < minDuration) {
                minDuration = val;
            }
            timeSum += val;
        }
        float mean = timeSum / allDurations.size();
        double rms = 0;
        for (Long val : allDurations) {
            rms += (val - mean) * (val - mean);
        }
        System.out.println("mean duration: " + mean);
        rms = (float) Math.sqrt(rms / allDurations.size());
        System.out.print("root mean square duration: " + (float) rms);
        System.out.println(" (" + (float) (rms / mean * 100) + " %)");
        System.out.println("min, max duration: " + minDuration + ", " + maxDuration);
    }

    public long start(int seconds) throws Exception {
        return start(seconds, OPTIMIZATION_FILE);
    }

    public long start(int seconds, String fileOrFolder) throws Exception {
        logger.info("process file:" + fileOrFolder);

        EventDao eDao = new EventDaoListImpl();
        PersonDao pDao = new PersonDaoListImpl();
        LocationDao lDao = new LocationDaoListImpl();
        FeatureDao fDao = new FeatureDaoListImpl();
        DataPool dataPool = new DataPoolImpl();
        dataPool.setDao(eDao);
        dataPool.setDao(fDao);
        dataPool.setDao(pDao);
        dataPool.setDao(lDao);

        NCPSettings algoSettings = new NCPSettings();
//        algoSettings.setCalculationIterations(seconds);
        algoSettings.setCalculationPeriodInSeconds(seconds);
        algoSettings.setMaxNodes(15000);
        algoSettings.setFinalDepth(4);
        algoSettings.setDepthMultiplier(1.5);

        // 1 hour
        DataPoolSettings settings = new DataPoolSettings();
        settings.setMilliSecondsPerTimeslot(60 * 60 * 1000L);
        settings.setNumberOfDays(5);
        // 9 hours per day
        settings.setTimeslotsPerDay(9);

        NoCollisionPrinciple ncpInstance = new NoCollisionPrinciple();
        ncpInstance.setRandom(new Random(1345678909876543219L));//123456L
        ncpInstance.setSettings(algoSettings);
        ncpInstance.setDataPoolSettings(settings);
        ncpInstance.setStatusBar(new SilentConsoleStatusBar());
        ncpInstance.setDataPool(dataPool);

        loadData(dataPool, new File(fileOrFolder), settings);

        long start = System.currentTimeMillis();
        ConstraintChecker.printStatistics(ncpInstance.doWork());

        return System.currentTimeMillis() - start;
    }

    public void loadData(DataPool dataPool, File fileOrFolder,
            DataPoolSettings settings) throws Exception {

        SetFactory factory = new SetFactoryImpl(settings);
        if (LOAD_BT) {
            ImportBTXml.overwriteSettings(settings);

            ImportBTXml importer = new ImportBTXml(dataPool, fileOrFolder);
            importer.setFactory(factory);
            importer.doWork();
        } else {
            InputStreamReader reader = new InputStreamReader(new FileInputStream(fileOrFolder));

            DataPool4Track2 dataPool4Track2 = new DataPool4Track2();
            dataPool4Track2.setFactory(factory);
            dataPool4Track2.setSettings(settings);
            dataPool4Track2.setDataPool(dataPool);
            dataPool4Track2.setSource(reader);

            dataPool4Track2.load();
        }
    }
}
