/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.io.xml;

import de.pannous.xvantage.core.writing.ObjectWriting;
import de.timefinder.data.DBInterface;
import de.timefinder.data.DataPool;
import de.timefinder.data.access.Dao;
import de.timefinder.data.set.BitRasterImpl;
import java.io.IOException;
import java.io.Writer;
import java.util.Collections;
import java.util.Map;
import javolution.util.FastMap;

/**
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class ExportXmlXvantage extends AbstractXvantageIO {

    private Writer writer;

    public ExportXmlXvantage(final DataPool tfPool, Writer writer) throws IOException {
        super(tfPool);
         // get objects from TimeFinder pool on demand
        this.xPool = new de.pannous.xvantage.core.DataPool() {

            @Override
            public Map<Class, Map<Long, Object>> getData() {
                Map<Class, Map<Long, Object>> map = FastMap.newInstance();
                for (Dao dao : tfPool.getDaos()) {
                    map.put(dao.getType(), dao.getMap());
                }
                return Collections.unmodifiableMap(map);
            }

            @Override
            public <T> Map<Long, T> getData(Class<T> clazz) {
                final Dao<T> dao = tfPool.getDao((Class) clazz);
                if (dao != null)
                    return (Map<Long, T>) dao.getMap();

                return null;
            }

            @Override
            public Long getId(Object object) {
                if (object instanceof DBInterface)
                    return ((DBInterface) object).getId();
                else
                    return null;
            }
        };
        this.writer = writer;
    }

    public void doWork() {
        xvantage.putWriting(BitRasterImpl.class, ObjectWriting.WRITING_TO_STRING);
        xvantage.saveObjects(writer, xPool);
        // TODO Helper.checkXsd("timefinder.xsd", writer);
    }
}
