/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.io.xml;

import de.pannous.xvantage.core.parsing.ObjectParsing;
import de.pannous.xvantage.core.parsing.Parsing;
import de.timefinder.algo.constraint.RasterConstraint;
import de.timefinder.data.DataPool;
import de.timefinder.data.Value;
import de.timefinder.data.access.Dao;
import de.timefinder.data.set.RasterEnum;
import de.timefinder.data.set.SetFactory;
import de.timefinder.data.set.WeekRaster;
import de.timefinder.data.set.WeekRasterImpl;
import java.io.IOException;
import java.io.Reader;
import java.util.BitSet;
import java.util.Map;
import java.util.Map.Entry;
import org.w3c.dom.Node;

/**
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class ImportXmlXvantage extends AbstractXvantageIO {

    private Reader reader;
    private SetFactory setFactory;
    private DataPool tfPool;

    public ImportXmlXvantage(DataPool tfPool, Reader reader) throws IOException {
        super(tfPool);
        this.tfPool = tfPool;
//        this.xPool = new DefaultDataPool();
        this.reader = reader;
    }

    public void setFactory(SetFactory setFactory) {
        this.setFactory = setFactory;
    }

    public void doWork() {
        xvantage.putParsing(WeekRasterImpl.class, new Parsing() {

            Parsing bSParsing = ObjectParsing.BITSET_PARSING;

            @Override
            public Object parse(Node node) {
                BitSet bs = (BitSet) bSParsing.parse(node);
                WeekRaster raster = setFactory.createWeekRaster();
                for (int ii = bs.nextSetBit(0); ii >= 0; ii = bs.nextSetBit(ii + 1)) {
                    raster.set(ii, RasterEnum.FORBIDDEN);
                }

                return raster;
            }
        });

        // TODO Helper.checkXsd("timefinder.xsd", reader);
        xPool = xvantage.readObjects(reader);
        for (Entry<Class, Map<Long, Object>> map : xPool.getData().entrySet()) {
            Dao dao = tfPool.getDao(map.getKey());
            if (dao != null)
                dao.attachAll(map.getValue().values());
        }

        for (Map<Long, Object> dataPerClass : xPool.getData().values()) {

            // if no raster was found initialize with empty rasters.
            // TODO only resources (person, location, feature) and events
            for (Object obj : dataPerClass.values()) {
                if (obj instanceof Value) {
                    Value val = ((Value) obj);
                    if (val.getFirstConstraint(RasterConstraint.class) == null) {
                        val.bindConstraint(new RasterConstraint(setFactory.createWeekRaster()));
                    }
                }
            }
        }
    }
}
