/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.ui;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.data.Event;
import de.timefinder.data.Feature;
import de.timefinder.data.Location;
import de.timefinder.data.Person;
import org.springframework.rules.constraint.Constraint;
import org.springframework.rules.Rules;
import org.springframework.rules.support.DefaultRulesSource;

/**
 * @see http://java.dzone.com/news/getting-further-with-spring-rc?page=0%2C6
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class Validation extends DefaultRulesSource {

    private Constraint NAME_VALIDATION = all(new Constraint[]{minLength(3), required()});
    private DataPoolSettings settings;

    public Validation() {
    }

    public void setSettings(DataPoolSettings settings) {
        this.settings = settings;
    }

    public void load() {
        addRules(new Rules(Person.class) {

            @Override
            protected void initRules() {
                add("name", NAME_VALIDATION);
            }
        });

        addRules(new Rules(Location.class) {

            @Override
            protected void initRules() {
                add("capacity", all(new Constraint[]{gte(1), required()}));
                add("name", NAME_VALIDATION);
            }
        });

        addRules(new Rules(Event.class) {

            @Override
            protected void initRules() {
                add("start", all(new Constraint[]{gte(0),
                            lt(settings.getTimeslotsPerWeek()), required()}));
                add("name", NAME_VALIDATION);
            }
        });

        addRules(new Rules(Feature.class) {

            @Override
            protected void initRules() {
                add("name", NAME_VALIDATION);
            }
        });
    }
}
