/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.ui.command;

import de.timefinder.data.DataPool;
import de.timefinder.data.Event;
import de.timefinder.data.Feature;
import de.timefinder.data.Location;
import de.timefinder.data.Person;
import javax.swing.JOptionPane;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class AnonymizeDataCommand extends TimeFinderCommand {

    public static final String ID = "anonymizeData";
    private DataPool dataPool;

    public AnonymizeDataCommand() {
        super(ID);
    }

    @Autowired
    public void setDataPool(DataPool dataPool) {
        this.dataPool = dataPool;
    }

    @Override
    protected void doOnce() {        
        if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(null,
                tr.get(ID + ".question"))) {
            
            int counter = 0;
            for (Feature f : dataPool.getDao(Feature.class).getAll()) {
                f.setName("feature" + counter++);
            }
            dataPool.getDao(Feature.class).refresh();

            counter = 0;
            for (Event ev : dataPool.getDao(Event.class).getAll()) {
                ev.setName("event " + counter++);
                ev.setDescription("");
            }
            dataPool.getDao(Event.class).refresh();

            counter = 0;
            for (Location loc : dataPool.getDao(Location.class).getAll()) {
                loc.setName("location " + counter++);
            }
            dataPool.getDao(Location.class).refresh();

            counter = 0;
            for (Person p : dataPool.getDao(Person.class).getAll()) {
                p.setName("person " + counter++);
                p.setBirthDay(null);
                p.setCity("");
                p.setFirstName("");
                p.setMobil("");
                p.setState("");
                p.setStreet("");
                p.setStreetNumber("");
                p.setTelephon("");
                p.setZip("");
            }
            dataPool.getDao(Person.class).refresh();

            logger.info("Anonymized all objects!");
        }
    }
}
