/*
 * Copyright 2008 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.timefinder.core.ui.command;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.algo.MyStatusBar;
import de.timefinder.algo.io.DataPool4Track2;
import de.timefinder.core.ui.StatusBarBridge;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.springframework.richclient.application.Application;
import org.springframework.richclient.application.ApplicationWindow;
import org.springframework.richclient.application.statusbar.StatusBar;

/**
 * This creates a command to open a file with the special format
 * from the International Timetabling Competition 2008.
 * 
 * @see de.timefinder.core.gstpl.data.DataPool4Track2
 * @author Peter Karich
 */
public class ImportTimFileCommand extends AbstractImportCommand {

    public static final String ID = "importTimFile";
    private DataPool4Track2 track2allData;

    /**
     * Creates a new {@code importTimFileCommand} with an id of {@value #ID}.
     */
    public ImportTimFileCommand() {
        super(ID);
    }

    public void setAllData(DataPool4Track2 dataPool) {
        this.track2allData = dataPool;
        setDataPool(dataPool.getDataPool());
    }

    /**
     * Starts a new frame and close the active window.
     * 
     * @see Application#close()
     */
    @Override
    protected void doOnce() {
        checkIfDataIsAlreadyAvailable();
        closeAllViews();

        DataPoolSettings settings = track2allData.getSettings();
        settings.setNumberOfDays(5);
        settings.setTimeslotsPerDay(9);

        int ret = JOptionPane.showConfirmDialog(null, tr.get(ID + ".question", 
                settings.getNumberOfDays(), settings.getTimeslotsPerDay()));
        if (ret != JOptionPane.OK_OPTION)
            return;

        ApplicationWindow aw = getApplicationWindow();
        final StatusBar bar = aw.getStatusBar();

        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(tr.get(ID + ".fileDialog"));
        int resultFC = fc.showOpenDialog(aw.getControl());
        if (resultFC != JFileChooser.APPROVE_OPTION) {
            bar.setMessage(tr.get(ID + ".noFileSelected"));
            return;
        }
        final File file = fc.getSelectedFile();
        if (!file.getName().endsWith(".tim")) {
            int resultOP = JOptionPane.showConfirmDialog(null, tr.get(ID + ".confirmation.message"),
                    tr.get(ID + ".confirmation.title"), JOptionPane.OK_CANCEL_OPTION);
            if (resultOP != JOptionPane.OK_OPTION) {
                return;
            }
        }

        bar.getProgressMonitor().taskStarted(tr.get(ID + ".startTask"), 100);

        MySwingWorker sw = new MySwingWorker(ID) {

            @Override
            protected void myconstruct(StatusBar bar) throws Exception {
                track2allData.setSource(file.getAbsolutePath());
                track2allData.setStatusBar(new StatusBarBridge(bar));
                track2allData.load();
                bar.getProgressMonitor().worked(90);
            }

            @Override
            protected void done() {
                openAllViews();
                bar.getProgressMonitor().worked(100);
                super.done();
            }
        };

        sw.execute();
    }
}
